# @(#) $Id: acld.bro 1042 2014-07-20 18:49:25Z leres $ (LBL)
#
#  Copyright (c) 2008, 2009, 2010, 2014
#  The Regents of the University of California. All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#      * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#      * Neither the name of the University nor the names of its contributors
#        may be used to endorse or promote products derived from this software
#        without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#  SUCH DAMAGE.
#

@load remote
@load notice

# Define new notices
redef enum Notice += {
	AcldDebug,
};

module acld;

type acld_request_rec: record {
	cmd: string;
	cookie: count;
	comment: string &optional;
};

type acld_reply_rec: record {
	cmd: string;
	cookie: count;
	timestamp: time;
	success: bool;
	comment: string &optional;
	payload: string &optional;
};

export {
	global acld_block: function(h: addr);
	global acld_blockhosthost: function(h1: addr, h2: addr);
	global acld_reply: event(rep: acld_reply_rec);
	global acld_request: event(msg: acld_request_rec);
	global acld_restorehosthost: function(h1: addr, h2: addr);
	global acld_unblock: function(h: addr);

	# Default acld host and port
	const acldaddr: addr = 127.0.0.1 &redef;
	const acldport: port = 47777/tcp &redef;
}

redef Remote::destinations += {
	["acld"] = [
		$host = acldaddr,
		$p = acldport,
		$class = "acld",
		$events = /acld::acld_reply/,
		$connect = T,
		$retry = 10 secs,
		$ssl=F
	]
};

global cookie: count = 0;

# Hook to re-block hosts at startup
event
bro_init()
{
	NOTICE([$note=AcldDebug, $msg=fmt("acld: acldaddr %s @ %s",
	    acldaddr, acldport) ]);
}

function
acld_block(h: addr)
{

NOTICE([$note=AcldDebug, $msg=fmt("acld_block: %s!", h) ]);

	event acld::acld_request([$cmd = fmt("drop %s", h), $cookie = cookie]);
	++cookie;
}

function
acld_blockhosthost(h:ddr, h2: addr addr)
{

NOTICE([$note=AcldDebug, $msg=fmt("acld_blockhosthost: %s %s!", h1, h2) ]);

	event acld::acld_request([$cmd = fmt("blockhosthost %s %s", h1, h2),
	    $cookie = cookie]);
	++cookie;
}

function
acld_restorehosthost(h1: addr, h2: addr)
{

NOTICE([$note=AcldDebug, $msg=fmt("acld_restorehosthost: %s %s!", h1, h2) ]);

	event acld::acld_request([$cmd = fmt("restorehosthost %s %s", h1, h2),
	    $cookie = cookie]);
	++cookie;
}

event
acld_reply(rep: acld_reply_rec)
{
	local str: string;

	if (rep$success)
		str = "success";
	else
		str = "fail";

	NOTICE([$note=AcldDebug, $msg=fmt("acld::reply(): \"%s\" %s (%d)",
		rep$cmd, str, rep$cookie) ]);
}

function
acld_unblock(h: addr)
{

NOTICE([$note=AcldDebug, $msg=fmt("acld_unblock: %s!", h) ]);
	event acld::acld_request([$cmd = fmt("restore %s", h),
	    $cookie = cookie]);
	++cookie;
}
