/*
 * Copyright (c) 2013, 2014
 * The Regents of the University of California. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University nor the names of its contributors
 *       may be used to endorse or promote products derived from this software
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] =
    "@(#) Copyright (c) 2013, 2014\n\
The Regents of the University of California.  All rights reserved.\n";
static const char rcsid[] =
    "@(#) $Id: rpctest.c 1058 2014-12-24 02:25:01Z leres $ (LBL)";
#endif

/*
 * rpctest - junoscript rpc test program
 */

#include <sys/types.h>

#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include "gnuc.h"
#ifdef HAVE_OS_PROTO_H
#include "os-proto.h"
#endif

#include "acld.h"
#include "child.h"
#include "client.h"
#include "io.h"
#include "rpc.h"

/* Globals */
int debug = 1;
const char *prog;
const char *logfile;
FILE *lf;
const char *configfn = "/usr/local/etc/acld.conf-mx5";
char hostname[64];
struct s2v astate2str[] = {
	{ NULL, 0 }
};
struct s2v cmd2req[] = {
	{ NULL, 0 }
};

/* Locals */
struct state state;

void
childaclcleanup(struct state *sp, struct req *rp)
{
}

void
clientsend(struct client *cl, struct req *rp)
{
	fprintf(stderr, "clientsend\n");
}

void
clientsenderr(struct client *cl, struct req *rp, const char *fmt, ...)
{
	fprintf(stderr, "clientsenderr\n");
	abort();
}

void
clientsendfmt(struct client *cl, struct req *rp, const char *fmt, ...)
{
	va_list ap;

	fprintf(stderr, "clientsendfmt: ");
	va_start(ap, fmt);
	vfprintf(stderr, fmt, ap);
	va_end(ap);
}

void
freereq(struct state *sp, struct req *rp)
{
	abort();
}

void
ioappendfmt(struct iobuf *ip, const char *fmt, ...)
{
	abort();
}

void
ioappendvfmt(struct iobuf *ip, const char *fmt, va_list ap)
{
	abort();
}

void
ioeat(struct iobuf *ip, size_t size)
{
	abort();
}

void
iofree(struct iobuf *ip)
{
	abort();
}

char *
iogetstr(struct iobuf *ip)
{
	abort();
}

int
iohaveline(register struct iobuf *ip)
{
	abort();
}

int
iohaverpcreply(struct iobuf *ip)
{
	abort();
}

int
main(int argc, char **argv)
{
	size_t cc;
	int fd;
	char *cp;
	const char *errmsg;
	xmlDoc *doc = NULL;
	xmlNode *root = NULL;
	char *buf;
	size_t bufsize;
	struct state *sp;
	struct cf *cf;
	struct client *cl;
	struct req *rp;
	struct array *dp, *dp2, *dp3;
	struct aclgroup *gp;
	struct acllist *ap;
	struct acl *al;
	int i, j, k;
	struct stat sbuf;

	log_lf.f = stderr;
	if (argv[0] == NULL)
		prog = "acld";
	else if ((cp = strrchr(argv[0], '/')) != NULL)
		prog = cp + 1;
	else
		prog = argv[0];

	if (argc != 2) {
		fprintf(stderr, "usage: %s xmlfile\n", prog);
		exit(1);
	}

	/* Check API for match with the DLL we are using */
	LIBXML_TEST_VERSION

	/* Get the hostname */
	if (gethostname(hostname, sizeof(hostname)) < 0)
		strcpy(hostname, "?");

	lf = stderr;
	sp = &state;
	sp->cf = parsecf(configfn);
	cf = sp->cf;

	fd = open(argv[1], O_RDONLY);
	if (fd < 0) {
		fprintf(stderr, "%s: open %s: %s\n",
		    prog, argv[1], strerror(errno));
		exit(1);
	}
	fprintf(stderr, "%s: opened %s\n", prog, argv[1]);
	if (fstat(fd, &sbuf) < 0) {
		fprintf(stderr, "%s: fstat %s: %s\n",
		    prog, argv[1], strerror(errno));
		exit(1);
	}
	bufsize = sbuf.st_size;
	buf = new(bufsize + 1, sizeof(*buf), "buf");
	cc = read(fd, buf, bufsize);
	if ((int)cc < 0) {
		fprintf(stderr, "%s: read %s: %s\n",
		    prog, argv[1], strerror(errno));
		exit(1);
	}
	if (cc != bufsize) {
		fprintf(stderr, "%s: short read %s: %d != %d\n",
		    prog, argv[1], (int)cc, (int)bufsize);
		exit(1);
	}
	(void)close(fd);

	doc = xmlReadMemory(buf, cc, "j.xml", NULL, 0);
	if (doc == NULL) {
		fprintf(stderr, "%s: xmlReadMemory failed\n", prog);
		exit(1);
	}

	root = xmlDocGetRootElement(doc);
	if (root == NULL) {
		fprintf(stderr, "%s: xmlDocGetRootElement failed\n", prog);
		exit(1);
	}

	cl = NULL;
	rp = NULL;
	errmsg = rpc_process(sp, cl, rp, root);

	if (errmsg != NULL)
		fprintf(stderr, "%s: rpc_process: %s\n", prog, errmsg);

	xmlFreeDoc(doc);
	xmlCleanupParser();

	/* Print what we parsed */
	dp = &cf->c_aclgroup_array;
	for (i = 0, gp = cf->c_aclgroup; i < dp->len; ++i, ++gp) {
		dp2 = &gp->acllist_array;
		for (j = 0, ap = gp->acllist; j < dp2->len; ++j, ++ap) {
			dp3 = &ap->acl_array;
			for (k = 0, al = ap->acl; k < dp3->len; ++k, ++al)
				fprintf(stderr, "%s:\t%s\n",
				    gp->name, aclformat(al));
		}
	}

	exit(0);
}

int
nullzeronetadd(struct cf *cf, const char *str)
{
	fprintf(stderr, "%s: nullzeronetadd: %s\n", prog, str);
	return (0);
}

void
stats_free(struct stats *st)
{
	abort();
}

void
stats_init(struct stats *st, size_t len, int secs, const char *what)
{
}

void
stats_sethiwater(struct stats *st, int len)
{
}

void
stats_setrate(struct stats *st)
{
}

int
timercheck(struct timer *tp)
{
	return (-1);
}

void
timerreset(struct timer *tp)
{
}

void
timerset(struct timer *tp, int secs)
{
}

void
whitelistread(struct cf *cf)
{
}

void
setastate(struct state *sp, enum acldstate astate)
{
}
